function [Node, Community]=ScaleFreeNetwork_CommunityStructure(N,M,m,n,gamma)
% This program generates a power-law degree distribution network with community structure and expected gamma
% N: population size
% M: number of communities
% m: average number of strong ties
% n: average number of weak ties
% gamma: power-law exponent 

if (m>=N/M) || (n>=N/M)
    disp('illegal input arguments of "n" or "m" ');
    return;
end

totoal_strong_ties=m*N/2;
totoal_weak_ties=n*N/2;
%% generate community and assign nodes into these communities
for i=1:M
    Community(i).nodes=(i-1)*(N/M)+1:i*(N/M); % the node list corresponding to the community
    Community(i).nums=N/M; % the number of nodes within the community
end
%% assign basic attribution to nodes
for i=1:N
    Node(i).nb=[]; % neighbor list
    Node(i).nums=0; % number of neighbors
    Node(i).st=[]; % strong tie list
    Node(i).stNums=0; % number of strong ties
    Node(i).wt=[]; % weak tie list
    Node(i).wtNums=0; % number of weak ties
end
%% generate connections
% 1.get nodes' weight
alpha=1/(gamma-1);
for i=1:N/M
    p(i)=1/(i^alpha);        
end
% 2.normalized node weights
sum_p=sum(p);
pp(1)=p(1)/sum_p;
for i=2:N/M
    pp(i)=pp(i-1)+p(i)/sum_p; % weight for strong tie    
end
varied_order_of_pp=randperm(N/M); % weight for weak tie
% 3.form strong ties
for i=1:totoal_strong_ties
    xsel=fix(rand(1)*M+1); % randomly select a community
    con=1;
    while con==1
        node_1_len=find(pp>rand(1)); 
        node_1=Community(xsel).nodes(node_1_len(1));
        node_2_len=find(pp>rand(1));
        node_2=Community(xsel).nodes(node_2_len(1));
        if (node_1~=node_2) && (sum(Node(node_1).nb==node_1)==0) % avoid self-loop and repeated connection
            Node(node_1).nb=[Node(node_1).nb;node_2];
            Node(node_1).nums=Node(node_1).nums+1;
            Node(node_1).st=[Node(node_1).st;node_2];
            Node(node_1).stNums=Node(node_1).stNums+1;
            Node(node_2).nb=[Node(node_2).nb;node_1];
            Node(node_2).nums=Node(node_2).nums+1;
            Node(node_2).st=[Node(node_2).st;node_1];
            Node(node_2).stNums=Node(node_2).stNums+1; 
            con=2;
        end
    end
end
% 4.form weak ties
for i=1:totoal_weak_ties
    con=1;
    while con==1
        community_1=fix(rand(1)*M+1);
        community_2=fix(rand(1)*M+1);
        while community_1==community_2
            community_2=fix(rand(1)*M+1);
        end
        node_1_len=find(pp>rand(1)); 
        node_1=Community(community_1).nodes(varied_order_of_pp(node_1_len(1)));
        node_2_len=find(pp>rand(1));
        node_2=Community(community_2).nodes(varied_order_of_pp(node_2_len(1)));
        if sum(Node(node_1).nb==node_1)==0 % avoid self-loop and repeated connection
            Node(node_1).nb=[Node(node_1).nb;node_2];
            Node(node_1).nums=Node(node_1).nums+1;
            Node(node_1).wt=[Node(node_1).wt;node_2];
            Node(node_1).wtNums=Node(node_1).wtNums+1;
            Node(node_2).nb=[Node(node_2).nb;node_1];
            Node(node_2).nums=Node(node_2).nums+1;
            Node(node_2).wt=[Node(node_2).wt;node_1];
            Node(node_2).wtNums=Node(node_2).wtNums+1; 
            con=2;
        end
    end
end